local AddonName, GGL = ...
local GUI = GGL.GUI
local Sets = GGL.Options.Sets
local Global = {}
GGL.Options.Sets_Item = Global

local L = GGL.Locale

-- lua
local ipairs = ipairs

-- WoW
local NUM_LE_ITEM_CLASSS = _G.NUM_LE_ITEM_CLASSES or _G.NUM_LE_ITEM_CLASSS or 19

-- Libs
local AceGUI = LibStub("AceGUI-3.0")

-- ##################
-- local
-- ##################
local Helper

local ITEM_TYPE_FILTER, SUB_ITEM_TYPE_FILTER = GGL.Templates.ItemFilter:GetFilterIgnores()

-- ##################
-- local helper
-- ##################
local function LoadOptionsString(str)
    local retC, ret = GGL:Deserialize(str)
    return retC and ret or {}
end

local function SaveDB()
    local t = Helper.Data["selectedDB"]
    for classID = 0, NUM_LE_ITEM_CLASSS-1 do
        if t[classID] and t[classID] ~= true then
            if not next(t[classID]) then
                t[classID] = nil
            end
        end
    end
    Helper:SaveDB()
end

local function CreateItemQualityList(content, callback)
    local data = Helper.Data
    local db = data["selectedDB"]
    local tabID = data["selectedTabID"]
    if tabID == "__config" then
        db = db["q"]
    else
        db = db["q"..tabID]
    end

    for qualityID = 0, #GGL.ITEM_QUALITY_INFO do
        local quality = GGL.ITEM_QUALITY_INFO[qualityID]
        local check = AceGUI:Create("CheckBox")
        check:SetLabel(quality.coloredDesc)
        if db and db[qualityID] then
            check:SetValue(false)
        else
            check:SetValue(true)
        end
        check:SetCallback("OnValueChanged", callback)
        check:SetUserData("qualityID", qualityID)
        content:AddChild(check)
    end
end

-- ##################
-- options
-- ##################
local function EnableButton_OnValueChanged(self, event, state)
    local data = Helper.Data
    local db = data["selectedDB"]
    local classID = data["selectedTabID"]

    if state == true then
        db[classID] = nil
        SaveDB()
    elseif state == false then
        db[classID] = true
        SaveDB()
    else -- nil
        db[classID] = {}
    end

    Helper:RefreshSelectedTab()
end

local function OverrideQualityCheck_OnValueChanged(self, event, state)
    local data = Helper.Data
    local db = data["selectedDB"]
    local classID = data["selectedTabID"]
    classID = "q"..classID

    if state then
        if not db[classID] then db[classID] = {} end
    else
        db[classID] = nil
        SaveDB()
    end
    Helper:RefreshSelectedTab()
end

local function ItemQualitySettingCheck_OnValueChanged(self, event, state)
    local qualityID = self:GetUserData("qualityID")
    local data = Helper.Data
    local db = data["selectedDB"]
    local tabID = data["selectedTabID"]

    if tabID == "__config" then
        if not state then
            if not db["q"] then db["q"] = {} end
        else
            if db["q"] and not next(db["q"]) then db["q"] = nil end
        end
        db = db["q"]
    else
        db = db["q"..tabID]
        if not db then return end
    end

    if state then
        db[qualityID] = nil
    else
        db[qualityID] = true
    end

    SaveDB()
end

local function SubClassCheck_OnValueChanged(self, event, state)
    local subClassID = self:GetUserData("subClassID")
    local data = Helper.Data
    if state then
        data["selectedDB"][ data["selectedTabID"] ][subClassID] = nil
    else
        data["selectedDB"][ data["selectedTabID"] ][subClassID] = true
    end
    SaveDB()
end

local function SetOnTabGroupSelected(content, db, tabID)
    -- top part
    local base = AceGUI:Create("SimpleGroup")
    base:SetFullWidth(true)
    base:SetLayout("Flow")
    content:AddChild(base)

    local enableCheck = AceGUI:Create("CheckBox")
    enableCheck:SetTriState(true) -- true, nil, false
    if db[tabID] == true then
        enableCheck:SetValue(false)
        enableCheck:SetLabel(L["|cffff0000Disabled|r"])
    elseif type(db[tabID]) == "table" then
        enableCheck:SetValue(nil)
        enableCheck:SetLabel(L["|cff00ff00Enabled|r"])
    else
        enableCheck:SetValue(true)
        enableCheck:SetLabel(L["|cff00ff00Enabled|r"])
    end
    enableCheck:SetCallback("OnValueChanged", EnableButton_OnValueChanged)
    base:AddChild(enableCheck)

    -- disabled skip from now
    if enableCheck:GetValue() == false then return end

    local newQualityFilter = AceGUI:Create("CheckBox")
    newQualityFilter:SetLabel(L["Override quality filter"])
    newQualityFilter:SetValue( (db["q"..tabID]) and true or false)
    newQualityFilter:SetCallback("OnValueChanged", OverrideQualityCheck_OnValueChanged)
    base:AddChild(newQualityFilter)

    -- new quality filter
    if newQualityFilter:GetValue() == true then
        local qBase = AceGUI:Create("SimpleGroup")
        qBase:SetFullWidth(true)
        --qBase:SetLayout("Flow")
        content:AddChild(qBase)

        CreateItemQualityList(qBase, ItemQualitySettingCheck_OnValueChanged)
    end

    -- on nil tristate
    if enableCheck:GetValue() == nil then
        local cont = AceGUI:Create("SimpleGroup")
        cont:SetFullWidth(true)
        cont:SetLayout("Flow")
        content:AddChild(cont)

        for subClassID = 0, 100 do
            if not (SUB_ITEM_TYPE_FILTER[tabID] and SUB_ITEM_TYPE_FILTER[tabID][subClassID]) then
                local subClassName = GetItemSubClassInfo(tabID, subClassID)
                if not subClassName or subClassName == "" then break end

                local check = AceGUI:Create("CheckBox")
                check:SetLabel(subClassName)
                check:SetValue(not db[tabID][subClassID])
                check:SetCallback("OnValueChanged", SubClassCheck_OnValueChanged)
                check:SetUserData("subClassID", subClassID)
                cont:AddChild(check)

            end
        end
    end
end

local function SoulboundCheck_OnValueChanged(self, event, state)
    if state then
        Helper.Data["selectedDB"]["s"] = nil
    else
        Helper.Data["selectedDB"]["s"] = true
    end
    SaveDB()
end

local function SetOnTabGroupConfigSelected(content, db, tabID)
    local base = AceGUI:Create("SimpleGroup")
    base:SetFullWidth(true)
    base:SetLayout("Flow")

    local sbCheck = AceGUI:Create("CheckBox")
    sbCheck:SetLabel(ITEM_SOULBOUND)
    sbCheck:SetImage(132853)
    sbCheck:SetValue(not db["s"])
    sbCheck:SetCallback("OnValueChanged", SoulboundCheck_OnValueChanged)
    base:AddChild(sbCheck)

    content:AddChild(base)

    CreateItemQualityList(content, ItemQualitySettingCheck_OnValueChanged)
end

local function Options(AceGUI, container)
    Helper = Sets:GetBaseGroup(container, GGL.SET_ID_LIST.Item)

    local tabs = {}
    for classID = 0, NUM_LE_ITEM_CLASSS-1 do
        if not ITEM_TYPE_FILTER[classID] then
            local className = GetItemClassInfo(classID)
            if className then
                tabs[#tabs+1] = {
                    value = classID,
                    text = className
                }
            end
        end
    end

    Helper:SetOnTabGroupSelected(SetOnTabGroupSelected)
    Helper:SetOnTabGroupConfigSelected(SetOnTabGroupConfigSelected)

    Helper:SetTabGroupContent(tabs, true)
end

-- Add into Options
GGL.Options:AddIntoTree(Options, L["Items"], "sets_items", "sets")